package com.example.survey.config;

import com.alibaba.fastjson.JSON;
import com.example.survey.dao.AuditDao;
import com.example.survey.entity.Audit;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.util.TokenUtil;
import com.example.survey.vo.ResultVO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import org.springframework.context.annotation.Configuration;
/**
 * @author Pope
 * 权限过滤器
 */
@Log4j2
@Configuration
public class AuthFilterConfig implements Filter {

    @Autowired
    private AuditDao auditDao;

    /**
     * 不需要token就能访问的路由
     */
    private static final Set<String> URIS = new HashSet<String>() {{
        add("/user/login");
    }};

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {

    }

    /**
     * 对请求进行过滤
     * 1.是否有token
     * 2.token所对应的权限是否能够访问对应请求
     */
    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;
        String method = request.getMethod();
        String uri = request.getRequestURI();
        String ip = request.getRemoteAddr();

        //生成审计记录
        Audit audit = new Audit();
        audit.setIp(ip);
        // Calendar calendar = Calendar.getInstance();
        // calendar.setTime(new Date());
        // calendar.add(Calendar.HOUR_OF_DAY, 8);
        audit.setTimestamp(System.currentTimeMillis());
        audit.setUri(uri);

        log.info(method + uri);

        //判断是否需要token
        if (URIS.contains(uri)) {
            audit.setUserPhone("登录操作，无需权限");
            auditDao.saveAudit(audit);

            filterChain.doFilter(servletRequest, servletResponse);
            return;
        }

        String token = request.getHeader("Authorization");
        if (uri.startsWith("/record/record2word")) {
            token = request.getParameter("token");
        }
        if (token == null) {
            log.error("请求无token");

            audit.setUserPhone("非法请求");
            auditDao.saveAudit(audit);

            returnJson(response, new ResultVO(ResultEnum.NO_TOKEN));
            return;
        }

        if (!TokenUtil.isPass(token, uri, method)) {
            log.error("非法token或权限不够");

            audit.setUserPhone("非法请求");
            auditDao.saveAudit(audit);

            returnJson(response, new ResultVO(ResultEnum.INSUFFICIENT_PRIVILEGE));
            return;
        }

        TokenUtil.refreshExpireTime(token);
        TokenUtil.refreshExpireTime(token + " : USER_PHONE");

        audit.setUserPhone((String) TokenUtil.get(token + " : USER_PHONE"));
        auditDao.saveAudit(audit);

        filterChain.doFilter(servletRequest, servletResponse);
    }

    @Override
    public void destroy() {

    }


    public static void returnJson(HttpServletResponse response, ResultVO resultVo) {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json;charset=utf-8");
        try (PrintWriter writer = response.getWriter()) {
            writer.print(JSON.toJSONString(resultVo));
        } catch (IOException e) {
            //由于统一拦截了异常所以不需要抛出
        }
    }
}
