package com.example.survey.config;

import lombok.extern.log4j.Log4j2;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.context.annotation.Configuration;

import java.util.Arrays;

/**
 * @author Pope
 */
@Log4j2
@Aspect
@Configuration
public class LogConfig {

    @Pointcut(value = "execution(* com.example.survey.controller.*.*(..))")
    public void controllerLog() {

    }

    @Pointcut(value = "execution(* com.example.survey.service.impl.*.*(..))")
    public void serviceLog() {

    }

    @Pointcut(value = "execution(* com.example.survey.dao.impl.*.*(..))")
    public void daoLog() {

    }


    @Around(value = "controllerLog()||serviceLog()||daoLog()")
    public Object before(ProceedingJoinPoint joinPoint) throws Throwable {
        log.info("class:{}, method:{}, params:{}", joinPoint.getTarget().getClass().getName(), joinPoint.getSignature().getName(), Arrays.toString(joinPoint.getArgs()));
        return joinPoint.proceed();
    }
}
