package com.example.survey.controller;

import com.example.survey.enumeration.AuthEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.service.RoleService;
import com.example.survey.dto.role.DeleteRoleDTO;
import com.example.survey.dto.role.CreateRoleDTO;
import com.example.survey.dto.role.ModifyRoleDTO;
import com.example.survey.vo.ResultVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Pope
 */
@RestController
@RequestMapping("/role")
public class RoleController {

    @Autowired
    private RoleService roleService;

    @PostMapping("/role")
    public ResultVO addRole(@RequestBody CreateRoleDTO createRoleDTO) {
        roleService.addRole(createRoleDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }

    @GetMapping("/roleList")
    public ResultVO getRole(@RequestParam(value = "pageSize", defaultValue = "30") int pageSize,
                            @RequestParam(value = "currentPage") int currentPage,
                            @RequestParam(value = "name", required = false) String name) {
        Map<String, Object> resultMap = new HashMap<>(16, 0.75F);
        resultMap.put("totalCount",roleService.countRole(name));
        resultMap.put("pageSize", pageSize);
        resultMap.put("currentPage", currentPage);
        resultMap.put("data",roleService.listRoleLimit(name, currentPage, pageSize));

        ResultVO resultVo = new ResultVO(ResultEnum.SUCCESS);
        resultVo.setData(resultMap);
        return resultVo;
    }

    @DeleteMapping("/role")
    public ResultVO deleteRole(@RequestBody DeleteRoleDTO deleteRoleDTO) {
        roleService.deleteRole(deleteRoleDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }

    @PutMapping("/role")
    public ResultVO modifyRole(@RequestBody ModifyRoleDTO modifyRoleDTO) {
        roleService.modifyRole(modifyRoleDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }

    @GetMapping("/authorityList")
    public ResultVO getAuthList() {
        ResultVO resultVo = new ResultVO(ResultEnum.SUCCESS);
        resultVo.setData(AuthEnum.getNameList());
        return resultVo;
    }


}
