package com.example.survey.dao;

import com.example.survey.entity.Role;
import com.example.survey.entity.User;

import java.util.List;

/**
 * @author Pope
 */
public interface UserDao {

    /**
     * 根据筛选条件分页查询用户
     *
     * @param username    用户名  模糊匹配
     * @param phoneNumber 电话号码  精确匹配
     * @param offset      偏移量
     * @param limit       数据量
     * @return 用户信息dto
     */
    List<User> listUserLimit(String username, String phoneNumber, int offset, int limit);

    /**
     * 根据电话号码判断是否存在用户
     *
     * @param phoneNumber 电话号码
     * @return 是否存在用户
     */
    boolean existUser(String phoneNumber);

    /**
     * 删除用户
     *
     * @param phone 电话号码
     */
    void deleteUser(String phone);


    /**
     * 从含有该角色的用户的角色列表中删除该用户
     *
     * @param role 角色
     */
    void clearRole(Role role);

    /**
     * 根据电话号码查询用户
     *
     * @param phone 电话
     * @return 用户
     */
    User selectUser(String phone);

    /**
     * 根据用户名与电话号码查询用户数量
     *
     * @param username 用户名
     * @param phone    电话号码
     * @return 用户数量
     */
    long countUser(String username, String phone);

    /**
     * 保存用户至数据库，如果是已有用户则更新信息
     *
     * @param user 用户
     */
    void saveUser(User user);
}
