package com.example.survey.dao.impl;

import com.example.survey.dao.MetaDataDao;
import com.example.survey.entity.MetaData;
import com.example.survey.enumeration.MetaDataTypeEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.MetaDataException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Repository;

import java.util.List;


/**
 * @author Pope
 */
@Repository
public class MetaDataDaoImpl implements MetaDataDao {

    @Autowired
    MongoTemplate mongoTemplate;

    @Override
    public boolean existMetaData(String name) {
        Query query = new Query(Criteria.where("name").is(name));
        return mongoTemplate.exists(query, MetaData.class);
    }

    @Override
    public MetaData selectMetaData(String name) {
        Query query = new Query(Criteria.where("name").is(name));
        return mongoTemplate.findOne(query, MetaData.class);
    }

    @Override
    public void saveMetaData(MetaData metaData) {
        try {
            mongoTemplate.save(metaData);
        } catch (Exception e) {
            throw new MetaDataException(ResultEnum.ALREADY_EXIST_METADATA);
        }
    }

    @Override
    public List<MetaData> listMetaDataLimit(String name, String type, int offset, int pageSize) {
        Criteria criteria = new Criteria();
        if (name != null) {
            criteria.and("name").regex(name);
        }
        if (type != null) {
            criteria.and("type").is(type);
        }
        Query query = new Query(criteria).skip(offset).limit(pageSize);
        return mongoTemplate.find(query, MetaData.class);

    }

    @Override
    public long countMetaData(String name, String type) {
        Criteria criteria = new Criteria();
        if (name != null) {
            criteria.and("name").regex(name);
        }
        if (type != null) {
            criteria.and("type").is(type);
        }
        Query query = new Query(criteria);
        return mongoTemplate.count(query, MetaData.class);
    }

    @Override
    public List<MetaData> selectAllMetaData() {
        return mongoTemplate.findAll(MetaData.class);
    }

    @Override
    public void deleteMetaData(String name) {
        Query query = new Query(Criteria.where("name").is(name));
        mongoTemplate.remove(query,MetaData.class);
    }
}
