package com.example.survey.dao.impl;

import com.example.survey.dao.ProjectDao;
import com.example.survey.entity.Project;
import com.example.survey.entity.Record;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.ProjectException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;


/**
 * @author Pope
 */
@Repository
public class ProjectDaoImpl implements ProjectDao {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public boolean existProject(String name) {
        Query query = new Query(Criteria.where("name").is(name));
        return mongoTemplate.exists(query, Project.class);
    }

    @Override
    public Project selectProject(String name) {
        Query query = new Query(Criteria.where("name").is(name));
        return mongoTemplate.findOne(query, Project.class);
    }

    @Override
    public void saveProject(Project project) {
        try {
            mongoTemplate.save(project);
        } catch (Exception e) {
            throw new ProjectException(ResultEnum.ALREADY_EXIST_PROJECT);
        }

    }

    @Override
    public List<Project> listProjectLimit(String name, long date_gt, long date_lt, int offset, int pageSize) {
        Criteria criteria = new Criteria();
        if (name != null) {
            criteria.and("name").regex(name);
        }
        if (date_gt != 0) {
            criteria.and("date").gt(date_gt);
        }
        if(date_lt != 0) {
            criteria.and("date").lt(date_lt);
        }
        Query query = new Query(criteria).skip(offset).limit(pageSize);
        return mongoTemplate.find(query, Project.class);
    }


    @Override
    public long countProject(String name, long date_gt, long date_lt) {
        Criteria criteria = new Criteria();
        if (name != null) {
            criteria.and("name").regex(name);
        }
        if (date_gt != 0) {
            criteria.and("date").gt(date_gt);
        }
        if(date_lt != 0) {
            criteria.and("date").lt(date_lt);
        }
        Query query = new Query(criteria);
        return mongoTemplate.count(query, Project.class);
    }

}
