package com.example.survey.dao.impl;

import com.example.survey.dao.MetaDataDao;
import com.example.survey.dao.RecordDao;
import com.example.survey.entity.MetaData;
import com.example.survey.entity.Record;
// import com.example.survey.entity.Respondent;
import com.example.survey.entity.User;
import com.example.survey.enumeration.RecordStateEnum;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Pope
 */
@Log4j2
@Repository
public class RecordDaoImpl implements RecordDao {

    @Autowired
    MongoTemplate mongoTemplate;

    @Autowired
    MetaDataDao metaDataDao;


    @Override
    public boolean existRecord(String uuid) {
        Criteria criteria = new Criteria()
                .and("uuid").is(uuid);
        Query query = new Query(criteria);
        return mongoTemplate.exists(query, Record.class);
    }

    @Override
    public Record getRecord(String uuid) {
        Criteria criteria = new Criteria()
                    .and("uuid").is(uuid);
        Query query = new Query(criteria);

        return mongoTemplate.findOne(query, Record.class);
    }

    @Override
    public void saveRecord(Record record) {
        mongoTemplate.save(record);
    }

    @Override
    public long countRecord(String idNumber, String state, String uuid, 
                            String templateName, String templateType, 
                            long submitTimeGt, long submitTimeLt, String projectName) {
        Criteria criteria = new Criteria();
        if (idNumber != null) {
            criteria.and("idNumber").is(idNumber);
        }
       
        if (state != null) {
            criteria.and("state").is(state);
        } 
        if (uuid != null) {
            criteria.and("uuid").is(uuid);
        }
        if (templateName != null) {
            MetaData metaData = metaDataDao.selectMetaData(templateName);
            if (metaData != null) {
                criteria.and("metaData.$id").is(metaData.getId());
            }
            else{
                return 0;
            }
        }

        if (templateType != null) {
            criteria.and("metaDataType").is(templateType);
        }

        if (submitTimeGt != 0) {
            criteria.and("submitTime").gt(submitTimeGt);
        }

        if (submitTimeLt != 0) {
            criteria.and("submitTime").lt(submitTimeLt);
        }


        if (projectName != null) {
            criteria.and("projectList").elemMatch(new Criteria().in(projectName));
        }

        Query query = new Query(criteria);
        return mongoTemplate.count(query, Record.class);
    }

    @Override
    public List<Record> listRecordLimit(String idNumber, String state, String uuid, 
                                        String templateName, String templateType, 
                                        long submitTimeGt, long submitTimeLt, String projectName,
                                        int offset, int pageSize) {
        Criteria criteria = new Criteria();
        if (idNumber != null) {
            criteria.and("idNumber").is(idNumber);
        }
       
        if (state != null) {
            criteria.and("state").is(state);
        } 
        else {
            criteria.and("state").nin(RecordStateEnum.FILED.getValue());
        }
        if (uuid != null) {
            criteria.and("uuid").is(uuid);
        }
        if (templateName != null) {
            MetaData metaData = metaDataDao.selectMetaData(templateName);
            if (metaData != null) {
                criteria.and("metaData.$id").is(metaData.getId());
            }
            else{
                return new ArrayList<Record>();
            }
        }

        if (templateType != null) {
            criteria.and("metaDataType").is(templateType);
        }

        if (submitTimeGt != 0) {
            criteria.and("submitTime").gt(submitTimeGt);
        }

        if (submitTimeLt != 0) {
            criteria.and("submitTime").lt(submitTimeLt);
        }


        if (projectName != null) {
            criteria.and("projectList").elemMatch(new Criteria().in(projectName));
        }

        Query query = new Query(criteria).skip(offset).limit(pageSize);
        return mongoTemplate.find(query, Record.class);
    }
}
