package com.example.survey.dao.impl;

import com.example.survey.dao.UserDao;
import com.example.survey.entity.Role;
import com.example.survey.entity.User;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.UserException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

import java.util.Iterator;
import java.util.List;
import java.util.Set;


/**
 * @author Pope
 */
@Log4j2
@Repository
public class UserDaoImpl implements UserDao {
    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public List<User> listUserLimit(String username, String phoneNumber, int offset, int limit) {
        Criteria criteria = new Criteria();

        //用户名模糊查询
        if (username != null) {
            criteria.and("username").regex(username);
        }

        //电话号码精确查询
        if (phoneNumber != null) {
            criteria.and("phone").is(phoneNumber);
        }

        Query query = new Query(criteria).skip(offset).limit(limit);
        return mongoTemplate.find(query, User.class);
    }

    @Override
    public boolean existUser(String phoneNumber) {
        Query query = new Query(Criteria.where("phone").is(phoneNumber));
        return mongoTemplate.exists(query, User.class);
    }

    @Override
    public void deleteUser(String phone) {
        Query query = new Query(Criteria.where("phone").is(phone));
        mongoTemplate.remove(query, User.class);
    }

    @Override
    public void clearRole(Role role) {
        Query query = new Query(Criteria.where("roleSet").elemMatch(Criteria.where("$id").is(role.getId())));
        List<User> users = mongoTemplate.find(query, User.class);
        for (User user : users) {
            Set<Role> roleSet = user.getRoleSet();
            Iterator<Role> it = roleSet.iterator();
            while (it.hasNext()) {
                if (role.getName().equals(it.next().getName())) {
                    it.remove();
                    break;
                }
            }
            Query userQuery = new Query(Criteria.where("phone").is(user.getPhone()));
            Update update = new Update().set("roleSet", roleSet);
            mongoTemplate.updateFirst(userQuery, update, User.class);
        }
    }


    @Override
    public User selectUser(String phone) {
        Query query = new Query(Criteria.where("phone").is(phone));
        return mongoTemplate.findOne(query, User.class);
    }

    @Override
    public long countUser(String username, String phone) {
        Criteria criteria = new Criteria();
        if (username != null) {
            criteria.and("username").regex(username);
        }
        if (phone != null) {
            criteria.and("phone").is(phone);
        }
        Query query = new Query(criteria);
        return mongoTemplate.count(query, User.class);
    }

    @Override
    public void saveUser(User user) {
        try {
            mongoTemplate.save(user);
        } catch (Exception e) {
            throw new UserException(ResultEnum.ALREADY_EXIST_USER);
        }
    }


}
