package com.example.survey.service;

import com.example.survey.dto.metaData.CreateMetaDataDTO;
import com.example.survey.dto.metaData.DeleteMetaDataDTO;
import com.example.survey.dto.metaData.ModifyMetaDataDTO;
import com.example.survey.entity.MetaData;
import com.example.survey.enumeration.MetaDataTypeEnum;
import com.example.survey.vo.MetaDataVO;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author Pope
 */
public interface MetaDataService {
    /**
     * 创建元数据
     *
     * @param createMetaDataDTO 元数据信息
     */
    void createMetaData(CreateMetaDataDTO createMetaDataDTO);

    /**
     * 根据筛选条件模糊查询元数据
     * @param name 元数据名
     * @param currentPage 当前页数
     * @param pageSize 页大小
     * @return 元数据名称
     */
    List<String> listMetaDataNameLimit(String name, String type, int currentPage, int pageSize);

    /**
     * 根据元数据名查询数量
     *
     * @param name 元数据名
     * @return 数量
     */
    long countMetaData(String name, String type);

    /**
     * 获取所有元数据名字的列表
     * @return 名字的列表
     */
    List<String> getNameList();

    /**
     * 修改元数据信息
     *
     * @param modifyMetaDataDTO 修改信息
     */
    void modifyMetaData(ModifyMetaDataDTO modifyMetaDataDTO);

    /**
     * @param name 名字
     * @return 元数据
     */
    MetaData getMetaData(String name);

    /**
     * 删除元数据
     *
     * @param deleteMetaDataDTO 删除信息
     */
    void deleteMetaData(DeleteMetaDataDTO deleteMetaDataDTO);
}
