package com.example.survey.service;

import com.example.survey.dto.record.*;
import com.example.survey.vo.RecordVO;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * @author Pope
 */
public interface RecordService {

  
    /**
     * 审核流调记录
     *
     * @param reviewRecordDTO 审核信息
     */
    void reviewRecord(ReviewRecordDTO reviewRecordDTO);

    /**
     * 修改已审核的流调记录
     *
     * @param modifyRecordDTO 修改信息
     */
    void modifyRecord(ModifyRecordDTO modifyRecordDTO);

    /**
     * 创建流调记录
     *
     * @param submitRecordDTO 流调记录信息
     */
    void createRecord(SubmitRecordDTO submitRecordDTO);

    /**
     * 根据筛选条件查询流调记录数量
     *
     * @param idNumber            调查对象身份证号
     * @param userPhone           分配的人员电话号码
     * @param projectName         项目名
     * @param state               流调记录状态
     * @param version             流调记录版本
     * @param questionnaireNumber 问卷编号
     * @return 流调记录数量
     */
    long countRecord(String idNumber, String state, String uuid, 
                    String templateName, String templateType, 
                    long submitTimeGt, long submitTimeLt, String projectName);
    /**
     * 根据筛选条件查询流调记录
     *
     * @param idNumber            调查对象身份证号
     * @param userPhone           分配的人员电话号码
     * @param projectName         项目名
     * @param state               流调记录状态
     * @param version             流调记录版本
     * @param questionnaireNumber 问卷编号
     * @param currentPage         当前页数，从0开始
     * @param pageSize            页大小
     * @return 流调记录VO
     */
    List<RecordVO> listRecordLimit(String idNumber, String state, String uuid, 
                                String templateName, String templateType, 
                                long submitTimeGt, long submitTimeLt,  String projectName, int offset, int pageSize);

    /**
     * 根据筛选条件查询流调记录的values
     *
     * @param projectName 项目名
     * @param idNumber 调查对象身份证号
     * @param version 版本号
     * @return 流调记录的values
     */
    Map<String,Object> getRecordValues(String uuid);

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 文件的可访问路径
     */
    String uploadFile(MultipartFile file);

    /**
     * 删除记录
     *
     * @param deleteRecordDTO 删除信息
     */
    void deleteRecord(DeleteRecordDTO deleteRecordDTO);

    /**
     * 项目导出为word文档
     *
     * @param idNumber 身份证号
     * @param projectName 项目名
     * @param response 响应
     */
    void record2word(String idNumber, HttpServletResponse response);


    /**
     * 修改流调记录所绑定元数据
     *
     * @param modifyMetaDataDTO 修改信息
     */
    void modifyMetaData(ModifyMetaDataDTO modifyMetaDataDTO);
}
