package com.example.survey.service;

import com.example.survey.dto.user.*;
import com.example.survey.vo.LoginVO;
import com.example.survey.vo.UserVO;

import java.util.List;

/**
 * @author Pope
 */
public interface UserService {
    /**
     * 根据登录信息判断是否登录
     *
     * @param loginDTO 登录信息
     * @return 用户信息  若登录失败则返回null
     */
    LoginVO matchAuth(LoginDTO loginDTO);

    /**
     * 注册用户
     *
     * @param createUserDTO 注册信息
     * @return 是否注册成功
     */
    void addUser(CreateUserDTO createUserDTO);

    /**
     * 修改用户权限
     *
     * @param modifyUserRoleDTO 新权限信息
     */
    void modifyRole(ModifyUserRoleDTO modifyUserRoleDTO);

    /**
     * 根据筛选条件分页查询用户
     *
     * @param username    用户名  模糊匹配
     * @param phoneNumber 电话号码  精确匹配
     * @param currentPage 当前页数
     * @param pageSize    页大小
     * @return 用户信息dto
     */
    List<UserVO> listUserLimit(String username, String phoneNumber, int currentPage, int pageSize);

    /**
     * 删除用户
     *
     * @param deleteUserDTO 删除信息
     */
    void deleteUser(DeleteUserDTO deleteUserDTO);

    /**
     * 修改用户信息
     *
     * @param modifyUserInfoDTO 用户信息
     */
    void modifyUserInfo(ModifyUserInfoDTO modifyUserInfoDTO);

    /**
     * 重置密码
     *
     * @param resetPwdDTO 用户信息
     */
    void resetPwd(ResetPwdDTO resetPwdDTO);

    /**
     * 根据用户名与电话号码查询用户数量
     *
     * @param username 用户名
     * @param phone    电话号码
     * @return 用户数量
     */
    long countUser(String username, String phone);
}
