package com.example.survey.service.impl;

import com.example.survey.dao.MetaDataDao;
import com.example.survey.dto.metaData.CreateMetaDataDTO;
import com.example.survey.dto.metaData.DeleteMetaDataDTO;
import com.example.survey.dto.metaData.ModifyMetaDataDTO;
import com.example.survey.entity.MetaData;
import com.example.survey.enumeration.MetaDataTypeEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.MetaDataException;
import com.example.survey.service.MetaDataService;
import com.example.survey.vo.MetaDataVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @author Pope
 */
@Service
public class MetaDataServiceImpl implements MetaDataService {

    @Autowired
    MetaDataDao metaDataDao;

    @Value("${file.path}")
    private String path;

    @Value("${file.url}")
    private String url;

    @Override
    public void createMetaData(CreateMetaDataDTO createMetaDataDTO) {
        if (metaDataDao.existMetaData(createMetaDataDTO.getName())) {
            throw new MetaDataException(ResultEnum.ALREADY_EXIST_METADATA);
        }
        MetaData metaData = new MetaData();
        metaData.setName(createMetaDataDTO.getName());
        metaData.setForm(createMetaDataDTO.getForm());
        metaData.setType(createMetaDataDTO.getType());
        // metaData.setFieldToNameList(createMetaDataDTO.getFieldToNameList());
        // metaData.setConfig(createMetaDataDTO.getConfig());
        // metaData.setWordTemplate(createMetaDataDTO.getWordTemplate());
        metaDataDao.saveMetaData(metaData);
    }

    @Override
    public List<String> listMetaDataNameLimit(String name, String type, int currentPage, int pageSize) {
        List<MetaData> metaDataList = metaDataDao.listMetaDataLimit(name, type, currentPage * pageSize, pageSize);
        if (metaDataList == null) {
            return new ArrayList<>();
        }
        return metaDataList.stream()
                .map(MetaData::getName)
                .collect(Collectors.toList());
    }

    @Override
    public long countMetaData(String name, String type) {
        return metaDataDao.countMetaData(name, type);
    }

    @Override
    public List<String> getNameList() {
        return metaDataDao.selectAllMetaData().stream().map(MetaData::getName).collect(Collectors.toList());
    }

    @Override
    public void modifyMetaData(ModifyMetaDataDTO modifyMetaDataDTO) {
        if (!metaDataDao.existMetaData(modifyMetaDataDTO.getName())) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        MetaData metaData = metaDataDao.selectMetaData(modifyMetaDataDTO.getName());
        metaData.setForm(modifyMetaDataDTO.getForm());
        if(modifyMetaDataDTO.getType() != null) {
            metaData.setType(modifyMetaDataDTO.getType());
        }
        
        // metaData.setFieldToNameList(modifyMetaDataDTO.getFieldToNameList());
        // metaData.setConfig(modifyMetaDataDTO.getConfig());
        metaDataDao.saveMetaData(metaData);
    }

    @Override
    public MetaData getMetaData(String name) {
        if (!metaDataDao.existMetaData(name)) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        return metaDataDao.selectMetaData(name);
    }

    @Override
    public void deleteMetaData(DeleteMetaDataDTO deleteMetaDataDTO) {
        if (!metaDataDao.existMetaData(deleteMetaDataDTO.getName())) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        metaDataDao.deleteMetaData(deleteMetaDataDTO.getName());
    }

}
