package com.example.survey.service.impl;

import com.example.survey.dao.MetaDataDao;
import com.example.survey.dao.ProjectDao;
import com.example.survey.dao.RecordDao;
import com.example.survey.dao.UserDao;
import com.example.survey.dto.project.AddRecordDTO;
import com.example.survey.dto.project.CreateProjectDTO;
import com.example.survey.dto.project.DeleteRecordDTO;
import com.example.survey.dto.project.ModifyProjectDTO;

import com.example.survey.entity.MetaData;
import com.example.survey.entity.Project;
import com.example.survey.entity.Record;
import com.example.survey.entity.User;
import com.example.survey.enumeration.ProjectStateEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.MetaDataException;
import com.example.survey.exception.ProjectException;
import com.example.survey.exception.RecordException;
import com.example.survey.exception.UserException;
import com.example.survey.service.ProjectService;
import com.example.survey.vo.ProjectVO;

// import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Pope
 */
@Service
public class ProjectServiceImpl implements ProjectService {

    @Autowired
    ProjectDao projectDao;

    @Autowired
    MetaDataDao metaDataDao;

    @Autowired
    UserDao userDao;

    @Autowired
    RecordDao recordDao;

    @Override
    public void createProject(CreateProjectDTO createProjectDTO) {

        Project project = new Project();
        project.setName(createProjectDTO.getName());
        project.setDate(System.currentTimeMillis());
        User user = userDao.selectUser(createProjectDTO.getPhone());
        project.setUser(user);
        projectDao.saveProject(project);
    }

    @Override
    public List<ProjectVO> listProjectLimit(String name, long date_gt, long date_lt, int currentPage, int pageSize) {
        return projectDao.listProjectLimit(name, date_gt, date_lt, currentPage * pageSize, pageSize).stream()
                .map(project -> {
                    ProjectVO projectVO = new ProjectVO();
                    projectVO.setName(project.getName());
                    projectVO.setDate(project.getDate());
                    projectVO.setUserName(project.getUser().getUsername());
                    projectVO.setUserPhone(project.getUser().getPhone());
                    projectVO.setCount(project.getCount());

                    return projectVO;
                }).collect(Collectors.toList());

    }

    @Override
    public long countRecord(String name) {
        if (!projectDao.existProject(name)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        
        return recordDao.countRecord(null, null, null, 
                                    null, null, 0, 0, name);
    }


    @Override
    public void modifyProject(ModifyProjectDTO modifyProjectDTO) {
      
        if (!projectDao.existProject(modifyProjectDTO.getName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        User user = userDao.selectUser(modifyProjectDTO.getPhone());
        Project project = projectDao.selectProject(modifyProjectDTO.getName());
        project.setUser(user);
        projectDao.saveProject(project);
    }

    @Override
    public long countProject(String name, long date_gt, long date_lt) {
        return projectDao.countProject(name, date_gt, date_lt);

    }

    @Override
    public void addRecord(AddRecordDTO addRecordDTO) {
        if(!projectDao.existProject(addRecordDTO.getName()))
        {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        if(!recordDao.existRecord(addRecordDTO.getUuid()))
        {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }

        Project project = projectDao.selectProject(addRecordDTO.getName());

        Record record = recordDao.getRecord(addRecordDTO.getUuid());
        List<String> t =  record.getProjectList();
        if(t == null) {
            t = new ArrayList<>();
        }
        if(!t.contains(project.getName()))
        {
            t.add(project.getName());
            record.setProjectList(t);
            recordDao.saveRecord(record);
            project.setCount(project.getCount() + 1);
            projectDao.saveProject(project);
        }
    }

    @Override
    public void deleteRecord(DeleteRecordDTO deleteRecordDTO) {
        if(!projectDao.existProject(deleteRecordDTO.getName()))
        {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        if(!recordDao.existRecord(deleteRecordDTO.getUuid()))
        {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }

        Project project = projectDao.selectProject(deleteRecordDTO.getName());

        Record record = recordDao.getRecord(deleteRecordDTO.getUuid());
        List<String> t =  record.getProjectList();
        if(t == null) {
            return;
        }
        if(t.contains(project.getName()))
        {
        //    List<String> t =  record.getProjectList();
            t.remove(project.getName());
            record.setProjectList(t);
            recordDao.saveRecord(record);
            project.setCount(project.getCount() - 1);
            projectDao.saveProject(project);

        }
        
    }

}
